#!/usr/bin/env ts-node

import * as FS from "fs";
import * as Path from "path";
import * as Utility from "util";
import * as Process from "process";

const Exist = FS.existsSync;

/***
 * Promisified Version of {@link FS.cp}
 * ---
 *
 * Asynchronously copies the entire directory structure from source to destination, including subdirectories and files.
 * - When copying a directory to another directory, globs are not supported.
 *
 * @experimental
 *
 * @param source {typeof import("fs").PathOrFileDescriptor} source path to copy.
 * @param target {typeof import("fs").PathOrFileDescriptor} destination path to copy to.
 * @returns {Promise<?>}
 *
 * @constructor
 *
 */

const Index = async (source, target) => {
    const $ = async () => await new Promise((resolve) => {
        FS.cp(Path.resolve(source), Path.resolve(target), {
                recursive: true,
                dereference: true,
                preserveTimestamps: false,
                errorOnExist: false,
                force: true
            }, () => resolve
        );
    })

    await $();
}

export { Index };
export default { Index };

const Arguments = Process.argv.splice(2);

const Source = (Arguments.includes("--source")) ? Arguments[Arguments.indexOf("--source") + 1] ?? null : null;
const Target = (Arguments.includes("--target")) ? Arguments[Arguments.indexOf("--target") + 1] ?? null : null;

(Target && Source) && (async () => {
    if (Exist(Source) === false) {
        throw Error("Source Directory Doesn't Exist");
    }

    await Index(Source, Target);
})();

(Target == null && Source == null) && (async () => {
    if (Exist(Source) === false) {
        throw Error("Package's \"node_modules\" Directory Doesn't Exist");
    }

    await Index("node_modules", "nodejs/node_modules")
})();
